<?php
define('VERSION', '2.0');
define('ABS', dirname(__FILE__));

function get_first_image_url($html){
	if (preg_match_all('/<img.+?src="(.+?)"/', $html, $matches)) {
		if ( count($matches[1]) ){
			foreach ($matches[1] as $src){
				if ( !is_exclude_image($src) ){
					return $src;
				}
			}
		}
	}
	return false;
}

function is_exclude_image( $src ){
	global $image_excludes;
	if ( !is_array($image_excludes) ){
		$image_excludes = explode(',', $image_excludes);
		array_map('trim', $image_excludes);
	}
	foreach ( $image_excludes as $pattern ){
		if ( preg_match("*".preg_quote($pattern)."*", $src) ){
			return true;
		}
	}
	return false;
}

function get_feed_thumbnail( $item, $cat = null ){
	if ( is_string($item) ){
		$item = get_cached_item($item);
	}
	if ( !is_null($cat) ){
		$fn = __FUNCTION__.'_'.$cat['id'];
		if ( is_callable($fn) ){
			$img = call_user_func($fn, $item);
			if ( $img ) return $img;
		}
	}
	if ( !($item instanceof SimplePie_Item ) ) return '/assets/img/no-image.png';
	$src = get_first_image_url( $item->get_content() );
	if ($src === false){
		$encs = $item->get_enclosures();
		if ( count($encs) ){
			foreach ( $encs as $enc ){
				if ( isset($enc->type) && preg_match('/image/',$enc->type) && isset($enc->link)){
					return $enc->link;
					break;
				}
			}
		}
		return '/assets/img/no-image.png';
	}
	return $src;
}

function get_feed_thumbnail_3( $item ){
	if ( is_string($item) ){
		$item = get_cached_item($item);
	}
	$data = $item->get_item_tags('http://search.yahoo.com/mrss/', 'content');
	if ( count($data) ){
		foreach ( $data as $d ){
			foreach ($d['attribs'] as $attr){
				if ( isset( $attr['type'] ) && preg_match('/image/', $attr['type']) && isset($attr['url'])){
					return $attr['url'];
				}
			}
		}
	}
	return false;
}

function get_feed_thumbnail_1( $item ){
	if ( is_string($item) ){
		$item = get_cached_item($item);
	}
	foreach ($item->data['child'] as $ns => $d){
		$data = $item->get_item_tags($ns, 'thumb_image');
		if ( isset( $data[0]['data'] ) ){
			return $data[0]['data'];
		}
	}
	return false;
}

function get_feed_thumbnail_16( $item ){
	if ( is_string($item) ){
		$item = get_cached_item($item);
	}
	if ( isset($item->data['child']) && is_array($item->data['child']) ){
		foreach ($item->data['child'] as $ns => $d){
			$data = $item->get_item_tags($ns, 'thumbnail');
			if ( is_array($data) ){
				$imgs = search_by_key($data, 'url');
				if ( count($imgs) ){
					return array_pop($imgs);
				}
			}
		}
	}
	return false;
}

function get_youtube_player($url, $width = 480, $height = 385){
	return "<iframe width=\"{$width}\" height=\"{$height}\" src=\"{$url}\" frameborder=\"0\" allowfullscreen></iframe>";
}

function search_in_feed( $array, $inv = null, $ink = null ){
	$founds = array();
	foreach ( $array as $k => $v ){
		if ( is_null($ink) || preg_match("/".preg_quote($ink)."/i", $k) ){
			if ( is_string($v) && (is_null($inv) || preg_match("/".preg_quote($inv)."/i", $v)) ){
				$tmp = array();
				$tmp[$k] = $v;
				$founds[]  = $tmp;
			} else if ( is_array($v) || is_object($v) ){
				$founds2 = search_in_feed($v, $inv, $ink);
				for( $i = 0; $i < count($founds2); $i++ ){
					$founds[] = $founds2[$i];
				}
			}
		}
	}
	return $founds;
}

function search_by_key( $array, $ink = null ){
	$founds = array();
	$q = array($array);
	while ( count($q) ){
		$top = array_shift($q);
		foreach ($top as $k => $v){
			if ( $k == $ink && is_string($v) ){
				array_push($founds, $top[$k]);
			}
			if ( is_array($v) || is_object($v) ){
				array_push($q, $top[$k]);
			}
		}
	}
	return $founds;
}

function get_cached_items($cache_dir, $excludes = array()){
	// files canbe load.
	$files = array();
	if ( file_exists($cache_dir) ){
		if ($handle = opendir($cache_dir)) {
			while (false !== ($file = readdir($handle))) {
				if (in_array($file, array('.', '..')) || strpos($file, '.')===0) continue;
				if (in_array($file, $excludes)) continue;
				if (is_file($cache_dir.$file)){
					array_push($files, $cache_dir.$file);
				}
			}
			closedir($handle);
		}
		rsort($files);
	}
	return $files;
}

function get_cached_item( $filename ){
	if ( file_exists($filename) ) {
		$handle = fopen($filename, "r");
		$content = fread($handle, filesize($filename));
		fclose($handle);
		return unserialize($content);
	}
	return false;
}

function clean_cache($cache_id) {
	$filename = cache_folder . $cache_id;
	return file_exists($filename) && @unlink($filename);
}

function save_cache($item, $cache_dir){
	if ( !file_exists($cache_dir) ){
		mkdir($cache_dir, 0755, true);
	}
	$cache_file = $cache_dir . md5($item->get_link());
	if ( !file_exists($cache_file) ){
		$handle = fopen($cache_file, 'a');
		$return = fwrite($handle, serialize($item)) ? 1 : 0;
		fclose($handle);
		return $return;
	}
	return 2;
}

function load_all_feeds( $urls = array() ){
	global $simplepie_cache_location;
	if ( !isset($simplepie_cache_location) ){
		$simplepie_cache_location = ABS.'/cache';
	}
	$all_items = array();
	
	foreach ($urls as $url){
		$excludes = array();
		$cache_dir = $simplepie_cache_location . '/' . md5($url) . '/';
		
		$feed = get_simplepie();
		$feed->set_feed_url($url);
		if ($feed->init()){
			$items = $feed->get_items();
			if ( count($items) ){
				foreach ($items as $j => $item){
					// var_dump($item);
					$state = save_cache($item, $cache_dir);
					if ( $state == 2 ){
						$excludes[] = md5( $item->get_link() );
					} else {
						global $cron_new_count;
						if ( isset($cron_new_count) ){
							$cron_new_count += $state;
						}
					}
					$all_items[] = $items[$j];
				}
			}
		}
		
		$cache_items = get_cached_items($cache_dir, $excludes);
		if ( count($cache_items) ){
			// log_message( $cache_items );
			foreach( $cache_items as $ci ){
				// $cached = get_cached_item($ci);
				$all_items[] = $ci;
			}
		}
	}
	return $all_items;
}

function log_message( $var, $logfile = '/system.log' ){
	$handle = fopen(ABS.$logfile, 'a');
	if ( is_string($var) ){
		fwrite($handle, $var.PHP_EOL);
	} else {
		ob_start();
		print_r($var);
		fwrite($handle, ob_get_clean().PHP_EOL);
	}
	fclose($handle);
}

function get_locale(){
	global $locale;
	if ( !isset($locale) || empty($locale) || !preg_match('/^[a-z]{2,2}-[A-Z]{2,2}$/', $locale) ){
		return 'en-US';
	}
	return $locale;
}

function seems_utf8($str) {
	$length = strlen($str);
	for ($i=0; $i < $length; $i++) {
		$c = ord($str[$i]);
		if ($c < 0x80) $n = 0; # 0bbbbbbb
		elseif (($c & 0xE0) == 0xC0) $n=1; # 110bbbbb
		elseif (($c & 0xF0) == 0xE0) $n=2; # 1110bbbb
		elseif (($c & 0xF8) == 0xF0) $n=3; # 11110bbb
		elseif (($c & 0xFC) == 0xF8) $n=4; # 111110bb
		elseif (($c & 0xFE) == 0xFC) $n=5; # 1111110b
		else return false; # Does not match any model
		for ($j=0; $j<$n; $j++) { # n bytes matching 10bbbbbb follow ?
			if ((++$i == $length) || ((ord($str[$i]) & 0xC0) != 0x80))
				return false;
		}
	}
	return true;
}

function utf8_uri_encode( $utf8_string, $length = 0 ) {
	$unicode = '';
	$values = array();
	$num_octets = 1;
	$unicode_length = 0;

	$string_length = strlen( $utf8_string );
	for ($i = 0; $i < $string_length; $i++ ) {

		$value = ord( $utf8_string[ $i ] );

		if ( $value < 128 ) {
			if ( $length && ( $unicode_length >= $length ) )
				break;
			$unicode .= chr($value);
			$unicode_length++;
		} else {
			if ( count( $values ) == 0 ) $num_octets = ( $value < 224 ) ? 2 : 3;

			$values[] = $value;

			if ( $length && ( $unicode_length + ($num_octets * 3) ) > $length )
				break;
			if ( count( $values ) == $num_octets ) {
				if ($num_octets == 3) {
					$unicode .= '%' . dechex($values[0]) . '%' . dechex($values[1]) . '%' . dechex($values[2]);
					$unicode_length += 9;
				} else {
					$unicode .= '%' . dechex($values[0]) . '%' . dechex($values[1]);
					$unicode_length += 6;
				}

				$values = array();
				$num_octets = 1;
			}
		}
	}

	return $unicode;
}

function sanitize_title_with_dashes( $title, $raw_title = '', $context = 'display') {
	$title = strip_tags($title);
	// Preserve escaped octets.
	$title = preg_replace('|%([a-fA-F0-9][a-fA-F0-9])|', '---$1---', $title);
	// Remove percent signs that are not part of an octet.
	$title = str_replace('%', '', $title);
	// Restore octets.
	$title = preg_replace('|---([a-fA-F0-9][a-fA-F0-9])---|', '%$1', $title);

	if (seems_utf8($title)) {
		if (function_exists('mb_strtolower')) {
			$title = mb_strtolower($title, 'UTF-8');
		}
		$title = utf8_uri_encode($title, 200);
	}

	$title = strtolower($title);
	$title = preg_replace('/&.+?;/', '', $title); // kill entities
	$title = str_replace('.', '-', $title);

	if ( 'save' == $context ) {
		// Convert nbsp, ndash and mdash to hyphens
		$title = str_replace( array( '%c2%a0', '%e2%80%93', '%e2%80%94' ), '-', $title );

		// Strip these characters entirely
		$title = str_replace( array(
			// iexcl and iquest
			'%c2%a1', '%c2%bf',
			// angle quotes
			'%c2%ab', '%c2%bb', '%e2%80%b9', '%e2%80%ba',
			// curly quotes
			'%e2%80%98', '%e2%80%99', '%e2%80%9c', '%e2%80%9d',
			'%e2%80%9a', '%e2%80%9b', '%e2%80%9e', '%e2%80%9f',
			// copy, reg, deg, hellip and trade
			'%c2%a9', '%c2%ae', '%c2%b0', '%e2%80%a6', '%e2%84%a2',
			// acute accents
			'%c2%b4', '%cb%8a', '%cc%81', '%cd%81',
			// grave accent, macron, caron
			'%cc%80', '%cc%84', '%cc%8c',
		), '', $title );

		// Convert times to x
		$title = str_replace( '%c3%97', 'x', $title );
	}

	$title = preg_replace('/[^%a-z0-9 _-]/', '', $title);
	$title = preg_replace('/\s+/', '-', $title);
	$title = preg_replace('|-+|', '-', $title);
	$title = trim($title, '-');

	return $title;
}

function remove_accents($string) {
	if ( !preg_match('/[\x80-\xff]/', $string) )
		return $string;

	if (seems_utf8($string)) {
		$chars = array(
				// Decompositions for Latin-1 Supplement
				chr(194).chr(170) => 'a', chr(194).chr(186) => 'o',
				chr(195).chr(128) => 'A', chr(195).chr(129) => 'A',
				chr(195).chr(130) => 'A', chr(195).chr(131) => 'A',
				chr(195).chr(132) => 'A', chr(195).chr(133) => 'A',
				chr(195).chr(134) => 'AE',chr(195).chr(135) => 'C',
				chr(195).chr(136) => 'E', chr(195).chr(137) => 'E',
				chr(195).chr(138) => 'E', chr(195).chr(139) => 'E',
				chr(195).chr(140) => 'I', chr(195).chr(141) => 'I',
				chr(195).chr(142) => 'I', chr(195).chr(143) => 'I',
				chr(195).chr(144) => 'D', chr(195).chr(145) => 'N',
				chr(195).chr(146) => 'O', chr(195).chr(147) => 'O',
				chr(195).chr(148) => 'O', chr(195).chr(149) => 'O',
				chr(195).chr(150) => 'O', chr(195).chr(153) => 'U',
				chr(195).chr(154) => 'U', chr(195).chr(155) => 'U',
				chr(195).chr(156) => 'U', chr(195).chr(157) => 'Y',
				chr(195).chr(158) => 'TH',chr(195).chr(159) => 's',
				chr(195).chr(160) => 'a', chr(195).chr(161) => 'a',
				chr(195).chr(162) => 'a', chr(195).chr(163) => 'a',
				chr(195).chr(164) => 'a', chr(195).chr(165) => 'a',
				chr(195).chr(166) => 'ae',chr(195).chr(167) => 'c',
				chr(195).chr(168) => 'e', chr(195).chr(169) => 'e',
				chr(195).chr(170) => 'e', chr(195).chr(171) => 'e',
				chr(195).chr(172) => 'i', chr(195).chr(173) => 'i',
				chr(195).chr(174) => 'i', chr(195).chr(175) => 'i',
				chr(195).chr(176) => 'd', chr(195).chr(177) => 'n',
				chr(195).chr(178) => 'o', chr(195).chr(179) => 'o',
				chr(195).chr(180) => 'o', chr(195).chr(181) => 'o',
				chr(195).chr(182) => 'o', chr(195).chr(184) => 'o',
				chr(195).chr(185) => 'u', chr(195).chr(186) => 'u',
				chr(195).chr(187) => 'u', chr(195).chr(188) => 'u',
				chr(195).chr(189) => 'y', chr(195).chr(190) => 'th',
				chr(195).chr(191) => 'y', chr(195).chr(152) => 'O',
				// Decompositions for Latin Extended-A
				chr(196).chr(128) => 'A', chr(196).chr(129) => 'a',
				chr(196).chr(130) => 'A', chr(196).chr(131) => 'a',
				chr(196).chr(132) => 'A', chr(196).chr(133) => 'a',
				chr(196).chr(134) => 'C', chr(196).chr(135) => 'c',
				chr(196).chr(136) => 'C', chr(196).chr(137) => 'c',
				chr(196).chr(138) => 'C', chr(196).chr(139) => 'c',
				chr(196).chr(140) => 'C', chr(196).chr(141) => 'c',
				chr(196).chr(142) => 'D', chr(196).chr(143) => 'd',
				chr(196).chr(144) => 'D', chr(196).chr(145) => 'd',
				chr(196).chr(146) => 'E', chr(196).chr(147) => 'e',
				chr(196).chr(148) => 'E', chr(196).chr(149) => 'e',
				chr(196).chr(150) => 'E', chr(196).chr(151) => 'e',
				chr(196).chr(152) => 'E', chr(196).chr(153) => 'e',
				chr(196).chr(154) => 'E', chr(196).chr(155) => 'e',
				chr(196).chr(156) => 'G', chr(196).chr(157) => 'g',
				chr(196).chr(158) => 'G', chr(196).chr(159) => 'g',
				chr(196).chr(160) => 'G', chr(196).chr(161) => 'g',
				chr(196).chr(162) => 'G', chr(196).chr(163) => 'g',
				chr(196).chr(164) => 'H', chr(196).chr(165) => 'h',
				chr(196).chr(166) => 'H', chr(196).chr(167) => 'h',
				chr(196).chr(168) => 'I', chr(196).chr(169) => 'i',
				chr(196).chr(170) => 'I', chr(196).chr(171) => 'i',
				chr(196).chr(172) => 'I', chr(196).chr(173) => 'i',
				chr(196).chr(174) => 'I', chr(196).chr(175) => 'i',
				chr(196).chr(176) => 'I', chr(196).chr(177) => 'i',
				chr(196).chr(178) => 'IJ',chr(196).chr(179) => 'ij',
				chr(196).chr(180) => 'J', chr(196).chr(181) => 'j',
				chr(196).chr(182) => 'K', chr(196).chr(183) => 'k',
				chr(196).chr(184) => 'k', chr(196).chr(185) => 'L',
				chr(196).chr(186) => 'l', chr(196).chr(187) => 'L',
				chr(196).chr(188) => 'l', chr(196).chr(189) => 'L',
				chr(196).chr(190) => 'l', chr(196).chr(191) => 'L',
				chr(197).chr(128) => 'l', chr(197).chr(129) => 'L',
				chr(197).chr(130) => 'l', chr(197).chr(131) => 'N',
				chr(197).chr(132) => 'n', chr(197).chr(133) => 'N',
				chr(197).chr(134) => 'n', chr(197).chr(135) => 'N',
				chr(197).chr(136) => 'n', chr(197).chr(137) => 'N',
				chr(197).chr(138) => 'n', chr(197).chr(139) => 'N',
				chr(197).chr(140) => 'O', chr(197).chr(141) => 'o',
				chr(197).chr(142) => 'O', chr(197).chr(143) => 'o',
				chr(197).chr(144) => 'O', chr(197).chr(145) => 'o',
				chr(197).chr(146) => 'OE',chr(197).chr(147) => 'oe',
				chr(197).chr(148) => 'R',chr(197).chr(149) => 'r',
				chr(197).chr(150) => 'R',chr(197).chr(151) => 'r',
				chr(197).chr(152) => 'R',chr(197).chr(153) => 'r',
				chr(197).chr(154) => 'S',chr(197).chr(155) => 's',
				chr(197).chr(156) => 'S',chr(197).chr(157) => 's',
				chr(197).chr(158) => 'S',chr(197).chr(159) => 's',
				chr(197).chr(160) => 'S', chr(197).chr(161) => 's',
				chr(197).chr(162) => 'T', chr(197).chr(163) => 't',
				chr(197).chr(164) => 'T', chr(197).chr(165) => 't',
				chr(197).chr(166) => 'T', chr(197).chr(167) => 't',
				chr(197).chr(168) => 'U', chr(197).chr(169) => 'u',
				chr(197).chr(170) => 'U', chr(197).chr(171) => 'u',
				chr(197).chr(172) => 'U', chr(197).chr(173) => 'u',
				chr(197).chr(174) => 'U', chr(197).chr(175) => 'u',
				chr(197).chr(176) => 'U', chr(197).chr(177) => 'u',
				chr(197).chr(178) => 'U', chr(197).chr(179) => 'u',
				chr(197).chr(180) => 'W', chr(197).chr(181) => 'w',
				chr(197).chr(182) => 'Y', chr(197).chr(183) => 'y',
				chr(197).chr(184) => 'Y', chr(197).chr(185) => 'Z',
				chr(197).chr(186) => 'z', chr(197).chr(187) => 'Z',
				chr(197).chr(188) => 'z', chr(197).chr(189) => 'Z',
				chr(197).chr(190) => 'z', chr(197).chr(191) => 's',
				// Decompositions for Latin Extended-B
				chr(200).chr(152) => 'S', chr(200).chr(153) => 's',
				chr(200).chr(154) => 'T', chr(200).chr(155) => 't',
				// Euro Sign
				chr(226).chr(130).chr(172) => 'E',
				// GBP (Pound) Sign
				chr(194).chr(163) => '',
				// Vowels with diacritic (Vietnamese)
				// unmarked
				chr(198).chr(160) => 'O', chr(198).chr(161) => 'o',
				chr(198).chr(175) => 'U', chr(198).chr(176) => 'u',
				// grave accent
				chr(225).chr(186).chr(166) => 'A', chr(225).chr(186).chr(167) => 'a',
				chr(225).chr(186).chr(176) => 'A', chr(225).chr(186).chr(177) => 'a',
				chr(225).chr(187).chr(128) => 'E', chr(225).chr(187).chr(129) => 'e',
				chr(225).chr(187).chr(146) => 'O', chr(225).chr(187).chr(147) => 'o',
				chr(225).chr(187).chr(156) => 'O', chr(225).chr(187).chr(157) => 'o',
				chr(225).chr(187).chr(170) => 'U', chr(225).chr(187).chr(171) => 'u',
				chr(225).chr(187).chr(178) => 'Y', chr(225).chr(187).chr(179) => 'y',
				// hook
				chr(225).chr(186).chr(162) => 'A', chr(225).chr(186).chr(163) => 'a',
				chr(225).chr(186).chr(168) => 'A', chr(225).chr(186).chr(169) => 'a',
				chr(225).chr(186).chr(178) => 'A', chr(225).chr(186).chr(179) => 'a',
				chr(225).chr(186).chr(186) => 'E', chr(225).chr(186).chr(187) => 'e',
				chr(225).chr(187).chr(130) => 'E', chr(225).chr(187).chr(131) => 'e',
				chr(225).chr(187).chr(136) => 'I', chr(225).chr(187).chr(137) => 'i',
				chr(225).chr(187).chr(142) => 'O', chr(225).chr(187).chr(143) => 'o',
				chr(225).chr(187).chr(148) => 'O', chr(225).chr(187).chr(149) => 'o',
				chr(225).chr(187).chr(158) => 'O', chr(225).chr(187).chr(159) => 'o',
				chr(225).chr(187).chr(166) => 'U', chr(225).chr(187).chr(167) => 'u',
				chr(225).chr(187).chr(172) => 'U', chr(225).chr(187).chr(173) => 'u',
				chr(225).chr(187).chr(182) => 'Y', chr(225).chr(187).chr(183) => 'y',
				// tilde
				chr(225).chr(186).chr(170) => 'A', chr(225).chr(186).chr(171) => 'a',
				chr(225).chr(186).chr(180) => 'A', chr(225).chr(186).chr(181) => 'a',
				chr(225).chr(186).chr(188) => 'E', chr(225).chr(186).chr(189) => 'e',
				chr(225).chr(187).chr(132) => 'E', chr(225).chr(187).chr(133) => 'e',
				chr(225).chr(187).chr(150) => 'O', chr(225).chr(187).chr(151) => 'o',
				chr(225).chr(187).chr(160) => 'O', chr(225).chr(187).chr(161) => 'o',
				chr(225).chr(187).chr(174) => 'U', chr(225).chr(187).chr(175) => 'u',
				chr(225).chr(187).chr(184) => 'Y', chr(225).chr(187).chr(185) => 'y',
				// acute accent
				chr(225).chr(186).chr(164) => 'A', chr(225).chr(186).chr(165) => 'a',
				chr(225).chr(186).chr(174) => 'A', chr(225).chr(186).chr(175) => 'a',
				chr(225).chr(186).chr(190) => 'E', chr(225).chr(186).chr(191) => 'e',
				chr(225).chr(187).chr(144) => 'O', chr(225).chr(187).chr(145) => 'o',
				chr(225).chr(187).chr(154) => 'O', chr(225).chr(187).chr(155) => 'o',
				chr(225).chr(187).chr(168) => 'U', chr(225).chr(187).chr(169) => 'u',
				// dot below
				chr(225).chr(186).chr(160) => 'A', chr(225).chr(186).chr(161) => 'a',
				chr(225).chr(186).chr(172) => 'A', chr(225).chr(186).chr(173) => 'a',
				chr(225).chr(186).chr(182) => 'A', chr(225).chr(186).chr(183) => 'a',
				chr(225).chr(186).chr(184) => 'E', chr(225).chr(186).chr(185) => 'e',
				chr(225).chr(187).chr(134) => 'E', chr(225).chr(187).chr(135) => 'e',
				chr(225).chr(187).chr(138) => 'I', chr(225).chr(187).chr(139) => 'i',
				chr(225).chr(187).chr(140) => 'O', chr(225).chr(187).chr(141) => 'o',
				chr(225).chr(187).chr(152) => 'O', chr(225).chr(187).chr(153) => 'o',
				chr(225).chr(187).chr(162) => 'O', chr(225).chr(187).chr(163) => 'o',
				chr(225).chr(187).chr(164) => 'U', chr(225).chr(187).chr(165) => 'u',
				chr(225).chr(187).chr(176) => 'U', chr(225).chr(187).chr(177) => 'u',
				chr(225).chr(187).chr(180) => 'Y', chr(225).chr(187).chr(181) => 'y',
				// Vowels with diacritic (Chinese, Hanyu Pinyin)
				chr(201).chr(145) => 'a',
				// macron
				chr(199).chr(149) => 'U', chr(199).chr(150) => 'u',
				// acute accent
				chr(199).chr(151) => 'U', chr(199).chr(152) => 'u',
				// caron
				chr(199).chr(141) => 'A', chr(199).chr(142) => 'a',
				chr(199).chr(143) => 'I', chr(199).chr(144) => 'i',
				chr(199).chr(145) => 'O', chr(199).chr(146) => 'o',
				chr(199).chr(147) => 'U', chr(199).chr(148) => 'u',
				chr(199).chr(153) => 'U', chr(199).chr(154) => 'u',
				// grave accent
				chr(199).chr(155) => 'U', chr(199).chr(156) => 'u',
		);

		// Used for locale-specific rules
		$locale = get_locale();

		if ( 'de-DE' == $locale ) {
			$chars[ chr(195).chr(132) ] = 'Ae';
			$chars[ chr(195).chr(164) ] = 'ae';
			$chars[ chr(195).chr(150) ] = 'Oe';
			$chars[ chr(195).chr(182) ] = 'oe';
			$chars[ chr(195).chr(156) ] = 'Ue';
			$chars[ chr(195).chr(188) ] = 'ue';
			$chars[ chr(195).chr(159) ] = 'ss';
		}

		$string = strtr($string, $chars);
	} else {
		// Assume ISO-8859-1 if not UTF-8
		$chars['in'] = chr(128).chr(131).chr(138).chr(142).chr(154).chr(158)
		.chr(159).chr(162).chr(165).chr(181).chr(192).chr(193).chr(194)
		.chr(195).chr(196).chr(197).chr(199).chr(200).chr(201).chr(202)
		.chr(203).chr(204).chr(205).chr(206).chr(207).chr(209).chr(210)
		.chr(211).chr(212).chr(213).chr(214).chr(216).chr(217).chr(218)
		.chr(219).chr(220).chr(221).chr(224).chr(225).chr(226).chr(227)
		.chr(228).chr(229).chr(231).chr(232).chr(233).chr(234).chr(235)
		.chr(236).chr(237).chr(238).chr(239).chr(241).chr(242).chr(243)
		.chr(244).chr(245).chr(246).chr(248).chr(249).chr(250).chr(251)
		.chr(252).chr(253).chr(255);

		$chars['out'] = "EfSZszYcYuAAAAAACEEEEIIIINOOOOOOUUUUYaaaaaaceeeeiiiinoooooouuuuyy";

		$string = strtr($string, $chars['in'], $chars['out']);
		$double_chars['in'] = array(chr(140), chr(156), chr(198), chr(208), chr(222), chr(223), chr(230), chr(240), chr(254));
		$double_chars['out'] = array('OE', 'oe', 'AE', 'DH', 'TH', 'ss', 'ae', 'dh', 'th');
		$string = str_replace($double_chars['in'], $double_chars['out'], $string);
	}

	return $string;
}


function get_seoname( $title, $fallback_title = '' ) {
	$raw_title = $title;

	$title = remove_accents($title);

	$title = sanitize_title_with_dashes($title, $raw_title, 'save');

	if ( '' === $title || false === $title )
		$title = $fallback_title;
	return preg_match('~%[0-9A-F]{2}~i', $title) ? urldecode($title) : $title;
}

function string_to_cloud($string){
	$cloudArray = array();
	$string = str_replace(array('"', "'", '(', ')', '[', ']', '&', '?', ':', ',', ';', '.' ), '', $string);
	$tags = preg_split("'\s+'",trim(preg_replace("'[^áéíóúa-z0-9‘][áéíóúa-z0-9‘]{1,3}[^áéíóúa-z0-9‘]'iu",' ',mb_strtolower(htmlspecialchars(stripslashes($string)),'UTF-8'))));
		foreach($tags as $tag){
			if(strlen($tag)>3)
				$cloudArray[$tag] = $tag;
		}
	return implode(",",$cloudArray);
}

function string_to_tags($string){
	$slug = get_seoname($string);

	$cloudArray = array();
	$tags = preg_split('/-/', $slug, -1, PREG_SPLIT_NO_EMPTY);
	foreach($tags as $tag){
		if(strlen($tag)>3)
			$cloudArray[$tag] = '<span class="label"><a href="/search/'.$tag.'" title="'.$tag.'">'.$tag.'</a></span>';
	}
	return implode(" ",$cloudArray);
}

function save_data($config_array, $storage = null){
	if ( is_null($storage) ){
		$storage = ABS.'/admin/config.json';
	}
	file_put_contents($storage, json_encode($config_array));
	return true;
}

function get_data( $storage = null ){
	global $data;
	if ( !isset($data) ){
		if ( is_null($storage) ){
			$storage = ABS.'/admin/config.json';
		}
		$data = json_decode( file_get_contents($storage), true );
	}
	return $data;
}

function force_download($file, $name = null) {
	// Setup
	if ( is_null($name) ){
		$name = basename($file);
	}
	$name = rawurldecode($name);
	$fileSize = filesize($file);

	// Disable Output Buffering
	@ob_end_clean();

	// IE Required
	if(ini_get('zlib.output_compression')) {
		ini_set('zlib.output_compression', 'Off');
	}

	// Send Headers
	header('Content-Type: application/json');
	header('Content-Disposition: attachment; filename="' . $name . '"');
	header('Content-Transfer-Encoding: binary');

	// Send Headers: Prevent Caching of File
	header('Cache-Control: private');
	header('Pragma: private');
	header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
	
	readfile($file);
}

function get_setting($name = '', $default = null){
	if ( is_null($name) || empty($name) ) return null;
	$data = get_data();
	if ( isset($data['tables']['settings']) ){
		foreach ( $data['tables']['settings'] as $element ){
			if ( isset($element['field_name']) && $element['field_name'] == $name ){
				return htmlspecialchars($element['field_value'], ENT_COMPAT, 'UTF-8');
			}
		}
	}
	return $default;
}

function get_raw_setting($name = '', $default = null){
	if ( is_null($name) || empty($name) ) return null;
	$data = get_data();
	if ( isset($data['tables']['settings']) ){
		foreach ( $data['tables']['settings'] as $element ){
			if ( isset($element['field_name']) && $element['field_name'] == $name ){
				return ($element['field_value']);
			}
		}
	}
	return $default;
}

function add_setting( $array ){
	$data = get_data();
	$has_change = false;
	$max_id = 0;
	if ( count($data['tables']['settings']) ){
		foreach ( $data['tables']['settings'] as $st){
			if ( $max_id < $st['id'] ){
				$max_id = $st['id'];
			}
		}
	}
	if ( is_string($array) ){
		if ( !isset($data['tables']['settings']) ){
			$data['tables']['settings'] = array();
		}
		$data['tables']['settings'][] = array( 'field_name' => $array, 'field_value' => '', 'id' => $max_id+1 );
		$has_change = true;
	} else if( is_array($array) ) {
		if ( isset($array['field_name']) ){
			if ( !isset($array['field_value']) ){
				$array['field_value'] = '';
			}
			$array['id'] = $max_id+1;
			$data['tables']['settings'][] = $array;
			$has_change = true;
		}
	}
	if ( $has_change ){
		save_data($data);
		return true;
	}
	return false;
}

function delete_setting( $name ){
	$data = get_data();
	$has_change = false;
	if ( is_string($name) ){
		if ( isset($data['tables']['settings']) ){
			foreach ($data['tables']['settings'] as $i => $element){
				if ( $name == $element['field_name'] ){
					unset( $data['tables']['settings'][$i] );
					$has_change = true;
				}
			}
		}
	}
	if ( $has_change ){
		save_data($data);
		return true;
	}
	return false;
}

function delete_directory( $path , $self = true ){
	if ( !file_exists($path) && !is_dir($path) ) return false;
	if ( $handle = opendir($path) ) {
		while ( false !== ($name = readdir($handle)) ) {
			if ($name[0] == '.') continue;
			$fullpath = $path . DIRECTORY_SEPARATOR . $name;
			if ( is_file($fullpath) ){
				@unlink($fullpath);
			} else if ( is_dir($fullpath) ){
				delete_directory($fullpath);
			}
		}
		closedir($handle);
	}
	if ( $self == true ) @rmdir($path);
}

function count_in_directory( $path ){
	if ( !file_exists($path) && !is_dir($path) ) return 0;
	$count = 0;
	if ( $handle = opendir($path) ) {
		while ( false !== ($name = readdir($handle)) ) {
			if ($name[0] == '.') continue;
			$fullpath = $path . DIRECTORY_SEPARATOR . $name;
			if ( is_file($fullpath) ){
				$count++;
			} else if ( is_dir($fullpath) ){
				$count += 10000;
			}
		}
		closedir($handle);
	}
	return $count;
}

function get_all_feeds( $id = 0 ){
	global $data;
	if ( !isset($ata) ) $data = get_data();
	
	$founds = array();
	if ( isset($data['tables']['categories']) ){
		foreach ( $data['tables']['categories'] as $cid => $cat ){
			if ( $id == 0 || $id == $cat['id'] ){
				foreach ( $cat['feeds'] as $feed ){
					$founds[] = $feed;
				}
			}
		}
	}
	return $founds;
}

function get_current_url() {
	$pageURL = 'http';
	if (isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"] == "on") {
		$pageURL .= "s";
	}
	$pageURL .= "://";
	if ($_SERVER["SERVER_PORT"] != "80") {
		$pageURL .= $_SERVER["SERVER_NAME"].":".$_SERVER["SERVER_PORT"].$_SERVER["REQUEST_URI"];
	} else {
		$pageURL .= $_SERVER["SERVER_NAME"].$_SERVER["REQUEST_URI"];
	}
	return $pageURL;
}

function get_command($index = 0){
	global $command;
	if ( !isset($command) || !is_array($command)) return null;
	if ( isset($command[$index]) ){
		return $command[$index];
	}
	return null;
}

function get_page(){
	global $pageName;
	if ( isset($pageName) ){
		return $pageName;
	}
	return null;
}

function is_logged_in(){
	return isset($_SESSION['id']);
}

function get_var($name){
	if ( array_key_exists($name, $_REQUEST)){
		return $_REQUEST[$name];
	}
	return null;
}

function get_simplepie($type = 'index'){
	require_once ABS.'/simplepie/autoloader.php';
	global $simplepie_cache_duration, $simplepie_item_limit, $simplepie_cache_location, $no_cache;
	if (true){
		if ( !isset($simplepie_cache_duration) ){
			$simplepie_cache_duration = 365 * 24 * 60 * 60;
		}
		if ( !isset($simplepie_cache_location) ){
			$simplepie_cache_location = ABS.'/cache';
		}
		if ( !isset($simplepie_item_limit) ){
			$simplepie_item_limit = 1000;
		}
	}
	$sp = new SimplePie();
	$sp->set_cache_location($simplepie_cache_location);
	$sp->set_cache_duration($simplepie_cache_duration);
	$sp->set_item_limit($simplepie_item_limit);
	$sp->remove_div(true);
	$sp->strip_htmltags(array_merge($sp->strip_htmltags, array('a','table','td','tr','script','b','nobr')));
	if ( isset($no_cache) && $no_cache ){
		$sp->set_cache_duration(0);
	}
	
	return $sp;
}

$category_index = array();
function get_category_indexed(){
	global $category_index;
	$categories = get_categories();
	if ( empty($category_index) && isset($categories) && is_array($categories) ){
		foreach( $categories as $kate => $category ){
			$i = 0;
			if ( !isset($category['feeds']) || empty($category['feeds']) ) continue;
			foreach ( $category['feeds'] as $feed ){
				if ( ($url = $feed['url']) && !isset($category_index[$url]) ){
					if ( !empty($feed['name']) ){
						$suffix = $feed['name'];
					} else {
						$suffix = $i;
					}
					$category_index[$url] = $category['name'].'-'.$suffix;
					$i++;
				}
			}
		}
	}
	return $category_index;
}

function pretty_name( $name ){
	$parts = explode('_', $name);
	return ucwords( implode(' ', $parts) );
}

function push_message($msg = '', $type = 'info'){
	if ( !isset($_SESSION['messages']) ){
		$_SESSION['messages'] = array();
	}
	$_SESSION['messages'][] = array(
			'message' => $msg,
			'type' => $type
	);
}

function print_messages(){
	if ( isset($_SESSION['messages']) && !empty($_SESSION['messages']) ){
		$messages = $_SESSION['messages'];
		if ( !is_array($messages)){
			echo '<div class="alert alert-info">';
			echo $messages;
			echo '</div>';
		} else {
			$groups = array();
			foreach ( $messages as $ames ){
				$type = isset($ames['type']) ? $ames['type'] : 'info';
				if ( !isset($groups[$type]) ){
					$groups[$type] = array();
				}
				$msg = isset($ames['message']) ? $ames['message'] : '' ;
				if (!$msg && isset($ames['msg'])) $msg = $ames['msg'];
				$groups[$type][] = $msg;
			}
			foreach ($groups as $type => $group){
				echo '<div class="alert alert-'.$type.'"><strong>'.ucfirst($type).'</strong><br/>';
				echo implode('<br/>', $group);
				echo '</div>';
			}
		}
		$_SESSION['messages'] = null;
		unset($_SESSION['messages']);
	}
}

function get_category_name( $id = 0 ){
	global $data;
	if (!isset($data)) $data = get_data();
	if ( isset($data['tables']['categories']) ){
		$cat_name = '';
		foreach ($data['tables']['categories'] as $cat){
			if ( $id == $cat['id'] ) {
				$cat_name = $cat['name'];
				break;
			}
		}
		return $cat_name;
	}
	return null;
}

function get_categories($id = 0){
	global $data;
	if (!isset($data)) $data = get_data();
	$cats = array();
	
	if ( isset($data['tables']['categories']) ){
		foreach ($data['tables']['categories'] as $cat){
			if ( (is_numeric($id)&&$id==0) || $id == $cat['id'] || $id == $cat['name'] || $id == strtolower($cat['name']) ){
				$cats[] = $cat;
			}
		}
	}
	return $cats;
}

function get_feed_urls($cid = 0){
	$urls = array();
	$cats = get_categories($cid);
	if ( count($cats) ){
		foreach ( $cats as $cat ){
			if (!empty($cat['feeds'])){
				foreach($cat['feeds'] as $feed){
					$urls[] = $feed['url'];
				}
			}
		}
	}
	return $urls;
}

function get_site_url( $parts = array(), $query = null ){
	global $site_url;
	$actual_url = $site_url;
	settype( $parts, 'array' );
	if ( count($parts) ){
		$actual_url .= implode('/', $parts) . '/';
	}
	if ( !is_null($query) ){
		$actual_url .= '?'.$query;
	}
	return $actual_url;
}

function get_category_by_seoname( $seoname ){
	$categories = get_categories();
	foreach ( $categories as $cid => $cat ){
		if ( get_seoname($cat['name']) == $seoname ){
			return $categories[$cid];
		}
	}
	return null;
}

function get_category_by_id( $id ){
	$cats = get_categories( $id );
	if ( !empty($cats) ){
		return array_pop($cats);
	}
	return null;
}

/* admin functions */

function get_model_storage( $type = '' ){
	return ABS.'/admin/models/'.$type.'.json';
}
function get_model( $type = '' ){
	global $models;
	if ( !empty($type) ){
		if ( !isset($models[$type]) ){
			// not load yet
			$models[$type] = array();
			if ( file_exists( $mf = get_model_storage($type) ) ){
				$models[$type] = load_model($mf);
			}
		}
		return $models[$type];
	}
	return array();
}

function load_model( $storage ){
	return json_decode( file_get_contents($storage), true );
}

function save_model( $data, $type ){
	if ( $type == 'page' ){
		$o = 0;
		foreach ($data as $i => $j){
			$data[$i]['ordering'] = $o++;
		}
	}
	return file_put_contents( get_model_storage($type), json_encode($data));
}

function get_view($view = ''){
	if ( !empty($view) && file_exists( $vf = ABS.'/admin/views/'.$view.'.php' ) ){
		return $vf;
	}
	return get_view('default');
}

function get_controller( $c = ''){
	if ( !empty($c) && file_exists( $cf = ABS.'/admin/controllers/'.$c.'.php' ) ){
		return $cf;
	}
	return false;
}

function get( $name, $default = null, $ref = null ){
	global $current;
	$search = isset($ref) ? $ref : $current;
	if ( is_array($search) && array_key_exists($name, $search) ){
		return $search[$name];
	} else if ( is_object($search) && property_exists($search, $name) ){
		return $search->$name;
	}
	return $default;
}

function get_page_by_id( $page_id ){
	$page = null;
	if ( $pages = get_model('page') ){
		foreach ( $pages as $i => $current_page ){
			if ( $page_id == $current_page['id'] ){
				$page = $pages[$i];
				break;
			}
		}
	}
	return $page;
}

function get_page_by_seoname( $page_seoname ){
	$page = null;
	if ( $pages = get_model('page') ){
		foreach ( $pages as $i => $current_page ){
			if ( isset($current_page['slug']) && $current_page['slug'] == $page_seoname ){
				$page = $pages[$i];
				break;
			}
		}
	}
	return $page;
}

function get_page_link( $page ){
	if ( isset($page['slug']) ){
		return get_site_url( array('page') ) . $page['slug'];
	}
	return '';
}

function get_admin_infomation(){
	if ( !defined('USERNAME') || !defined('PASSWORD') ){
		$data = get_data();
		if ( isset($data['tables']['users']) ){
			define('USERNAME', $data['tables']['users']['username']);
			define('PASSWORD', $data['tables']['users']['password']);
		}
	}
	return defined('USERNAME') && defined('PASSWORD');
}

function display_banner( $id ){
	$banners = get_model('banner');
	if ( count($banners) ){
		foreach ( $banners as $i => $banner ){
			if ( $id == $banner['id'] || $id == $banner['title'] || $id == $banner['content'] ){
				echo '<!-- Start Ads: ' . $banner['title'] . ' -->';
				echo $banner['content'];
				echo '<!-- End Ads: ' . $banner['title'] . ' -->';
			}
		}
	}
	return true;
}

function image_process(&$imageown, $files){
	if ( !empty($files) ){
		$image = null;
		foreach ($files as $name => $info){
			if ( isset($info['name']) && !empty($info['name']) && @$info['size']>0){
				$image = $files[$name];
			}
		}
		if ( !is_null($image) ){
			$image_dir = get_setting('image_directory', ABS.'/media');
			$newfilename = sanitize_file_name($image['name']);
			if ( check_dir($image_dir) ){
				$newfile = $image_dir.'/'.$newfilename;
				if ( file_exists($newfile) && !@unlink($newfile) ){
					return false;
				}
				$imageown['image'] = 'media/'.$newfilename;
				return move_uploaded_file($image['tmp_name'], $newfile);
			}
		}
	} else {
		$imageown['image'] = '';
	}
}

function front_page_id_callback( $data ){
	$page_array = array('-1' => 'Default');
	if ( $pages = get_model( 'page' ) ){
		foreach ($pages as $page){
			$page_array[$page['id']] = $page['title'];
		}
	}
	return array(
			'field_name' => 'front_page_id',
			'field_type' => 'select',
			'field_choices' => $page_array,
			'field_value' => get('field_value', '-1', @$data['setting'])
	);
}

function default_layout_callback( $data ){
	return array(
			'field_name' => 'default_layout',
			'field_type' => 'select',
			'field_choices' => array(
					'grid' => 'Grid',
					'list' => 'List'
			),
			'field_value' => get('field_value', 'list', @$data['setting'])
	);
}

function enable_rtl_callback( $data ){
	return array(
			'field_name' => 'enable_rtl',
			'field_type' => 'select',
			'field_choices' => array(
					'0' => 'No',
					'1' => 'Yes'
			),
			'field_value' => get('field_value', '0', @$data['setting'])
	);
}

function preview_frame_callback( $data ){
	return array(
			'field_name' => 'preview_frame',
			'field_type' => 'select',
			'field_choices' => array(
					'1' => 'Enabled',
					'0' => 'Disabled'
			),
			'field_value' => get('field_value', '0', @$data['setting'])
	);
}


?>